use crate::util::pack::{
    pack_u16,
    unpack_u16,
};
use game::{
    history::Play,
    util::mask,
};

use super::Packet;

#[derive(Clone)]
pub struct PacketGamePlay {
    pub status:u16,
    pub turn:u16,
    pub play:Play,
}
impl PacketGamePlay {
    pub fn new() -> Self
    {
        Self {
            status:0,
            turn:0,
            play:Play::new(),
        }
    }
}
impl Packet for PacketGamePlay {
    type Data = Self;

    fn decode(data:&Vec<u8>, index:&mut usize) -> Result<Self::Data, ()>
    {
        let mut result = Self::new();

        result.status = unpack_u16(data, index);
        result.turn = unpack_u16(data, index);

        let play = unpack_u16(data, index) as u32;
        result.play.source = (play & mask(4, 0)) as u8;
        result.play.from = ((play & mask(6, 4)) >> 4) as u8;
        result.play.to = ((play & mask(6, 10)) >> 10) as u8;

        Ok(result)
    }
    
    fn encode(&self) -> Vec<u8>
    {
        let mut data = 0;
        data |= self.play.source as u16;
        data |= (self.play.from as u16) << 4;
        data |= (self.play.to as u16) << 10;
        [
            pack_u16(self.status),
            pack_u16(self.turn),
            pack_u16(data),
        ].concat()
    }
}
